#include <stdio.h>
#include <stdlib.h>

#include <mpi.h>

int main (int argc, char ** argv)
{
  int n, i, size, rank;
  double pi, spi, s, d, x;

  double * gbuf;

  MPI_Init (&argc, &argv);
  MPI_Comm_size (MPI_COMM_WORLD, &size);
  MPI_Comm_rank (MPI_COMM_WORLD, &rank);

  fprintf (stdout, "I am proces %3d of %3d\n", rank, size);
  fflush(stdout);


  if (argc != 2)
    n = 10;
  else
    n = atoi(argv[1]);

  d = 1.0/n;
  s = 0.0;

  for (i=rank+1; i<=n; i+=size) {
    x = (i-0.5)*d;
    s += 4.0/(1.0+x*x);
  }
  spi = d*s;

  gbuf = (double *) malloc (size * sizeof (double));

  MPI_Gather(&spi, 1, MPI_DOUBLE, gbuf, 1, MPI_DOUBLE, 0, MPI_COMM_WORLD);

  if (rank == 0) {
    pi = 0.0;
    for (i=0; i<size; i++) {
      fprintf ( stdout, "  %d = %.23f...\n", i, gbuf[i]);
      pi += gbuf[i];
    }

    fprintf ( stdout, "%d computed pi value = %.23f...\n", rank, pi);
  }

  free (gbuf);

  MPI_Finalize ();
 
  return 0;
} 
